import decimal
import json
from django.shortcuts import render, redirect
from django.urls import reverse
from .forms import *
from django.contrib import messages
from django.http import JsonResponse
from django.template.response import TemplateResponse

# Create your views here.
def staffDirectory(request):
    
    if request.method == "POST":
        form = StaffForm(request.POST, request.FILES)
        if form.is_valid():
            # image = request.FILES.get('image')
            staff = form.save(commit=False)
            
            staff_id = form.cleaned_data['role']
            
            staff_id = str(staff_id)[:2] 
            
            staff.save()
            
            Staff.objects.filter(id=staff.id).update(staff_id=str(staff_id) + str(staff.id))
            #staff.staff_id = staff_id
            messages.success(request, "Staff added successfully")
        else:
            messages.error(request, form.errors)
    form = StaffForm()
    staff = Staff.objects.all()
    designation =  Designation.objects.all()
    department = Department.objects.all()
    role = Role.objects.all()
    
    # for i in staff:
    #     print(i.image)
    
    media_url =  "http://127.0.0.1:8000/media/"
    context = {'form':form, 'staff':staff, 'media_url':media_url, 'designations':designation, 'departments':department, 'roles':role}
    return render(request, 'staffDirectory.html', context)




def department(request):
    
    if request.method == "POST":
        form = DepartmentForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, "Department added successfully")
        else:
            return
    form = DepartmentForm()
    
    departments = Department.objects.all()
    # print(departments)
    context = {'form':form, 'departments':departments}
    return render(request, 'department.html', context)






def designation(request):
    
    if request.method == "POST":
        form = DesignationForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, "Designation added successfully")
        else:
            return
    form = DesignationForm()
    
    designations = Designation.objects.all()
    context = {'form':form, 'designations':designations}
    return render(request, 'designation.html', context)




def role(request):
    
    if request.method == "POST":
        form = RoleForm(request.POST)
        if form.is_valid():
            form.save()
            messages.success(request, "Role added successfully")
        else:
            return
    form = RoleForm()
    
    roles = Role.objects.all()
    context = {'form':form, 'roles':roles}
    
    return render(request, 'roles.html', context)





def payroll(request):
    
    roles = Role.objects.all()
    YEAR_CHOICES = [ r for r in range(datetime.date.today().year-1, datetime.date.today().year+1)]
    
    context = {'roles':roles, 'YEAR_CHOICES':YEAR_CHOICES}
    return render(request, "payroll.html", context)




def payrollData(request):
     
    data = json.loads(request.body)

    role = data['role']
    month = data['month']
    year = data['year']
    
    staff = Staff.objects.filter(role=role)

    monthly_pay = []
    for sta in staff:
        payroll = Payroll.objects.filter(month=month, year=year, staff=sta)
        
        if payroll:

            paid = [sta.staff_id, sta.name, sta.department.department,  sta.contact, "Generated", payroll.first().id]
            monthly_pay.append(paid)
        else:

            not_paid = [sta.staff_id, sta.name, sta.department.department,  sta.contact, "Not Generated", 0]
            monthly_pay.append(not_paid)
    
    # print(monthly_pay)
    return JsonResponse(list(monthly_pay), safe=False)






def getEmployeeInfo(request):
    
    id = request.GET.get('id')
    
    staff = Staff.objects.filter(staff_id=id).last()
    
    context = {}
    if staff:
        context['id'] = staff.staff_id
        context['name'] = staff.name
        context['role'] = staff.role.role
        context['department'] = staff.department.department
        context['designation'] = staff.designation.designation
        context['basic_salary'] = staff.basic_salary
        context['phone'] = staff.contact
        # if staff.image:
        #     context['image'] = str(staff.image.url)
    else:
        context['id'] = "Not Found"
        context['name'] = "Not Found"
        context['role'] = "Not Found"
        context['department'] = "Not Found"
        context['designation'] = "Not Found"
        context['basic_salary'] = "Not Found"
        # context['image'] = "Not Found"
        

    # print(context)
    return JsonResponse(context, safe=False)







def savePayroll(request):
    
    data = json.loads(request.body)

    penalty = data['penalty'] 
    basic_salary = data['basic_salary']
    advance_salary = data['advance_salary']
    deduction = data['deduction']
    bonus = data['bonus']
    allowance = data['allowance']
    deductions = data['deductions']
    net_salary = data['net_salary']
    staff_id = data['staff_id']
    tax = data['tax']
    rent = data['rent']
    year = data['payroll_year']
    month = data['payroll_month']
    
    # print(year, month)
    
    
    staff = Staff.objects.filter(staff_id=staff_id).first()
    
    if not staff:
        return JsonResponse({'Error': 'Staff ID cannot be found'}, status=404, safe=False)
    
    
    payroll = Payroll(  
                    staff = staff,
                    basic_salary = basic_salary,
                    tax = tax,
                    month = month,
                    year = year,
                    net_salary = net_salary,
                    rent = rent,
                    penalty = penalty,
                    advance_salary = advance_salary,
                    deduction = deduction,
                    bonus = bonus,
                    allowance = allowance,
                    total_deductions = deductions,
                    )
    payroll.save()
    

    return JsonResponse({'Success':'Success'}, safe=False)





def saveEditedPayroll(request):
    
    data = json.loads(request.body)

    penalty = data['penalty'] 
    basic_salary = data['basic_salary']
    advance_salary = data['advance_salary']
    deduction = data['deduction']
    bonus = data['bonus']
    allowance = data['allowance']
    deductions = data['deductions']
    net_salary = data['net_salary']
    tax = data['tax']
    rent = data['rent']
    id = data['id']
    # print(type(id))
    # print(rent)
    # print(net_salary)
    Payroll.objects.filter(id=id).update(
                    basic_salary = basic_salary,
                    tax = tax,
                    net_salary = net_salary,
                    rent = rent,
                    penalty = penalty,
                    advance_salary = advance_salary,
                    deduction = deduction,
                    bonus = bonus,
                    allowance = allowance,
                    total_deductions = deductions,
    )
    
    
    return JsonResponse({'Success':'Success'}, safe=False)






def editPayroll(request):
    
    id = request.GET.get('id')
    
    payroll = Payroll.objects.filter(id=id).last()
    # print(payroll.id)
    
    staff = Staff.objects.filter(id = payroll.staff.id).last()
    # print(staff)
    
    context = {}
    
    if staff and payroll:
        context['id'] = payroll.id
        context['name'] = staff.name
        context['role'] = staff.role.role
        context['department'] = staff.department.department
        context['designation'] = staff.designation.designation
        context['basic_salary'] = staff.basic_salary
        context['phone'] = staff.contact
        
        context['penalty']  = payroll.penalty
        context['basic_salary']  = payroll.basic_salary
        context['advance_salary'] = payroll.advance_salary
        context['deduction'] = payroll.deduction
        context['bonus'] = payroll.bonus
        context['allowance'] = payroll.allowance
        context['deductions'] = payroll.total_deductions
        context['net_salary'] = payroll.net_salary
        context['tax'] = payroll.tax
        context['rent'] = payroll.rent
        context['payroll_year'] = payroll.year
        context['payroll_month'] = payroll.month
        context['staff_id'] = staff.staff_id
        
        
        # if staff.image:
        #     context['image'] = str(staff.image.url)
            
    else:
        context['id'] = "Not Found"
        context['name'] = "Not Found"
        context['role'] = "Not Found"
        context['department'] = "Not Found"
        context['designation'] = "Not Found"
        context['basic_salary'] = "Not Found"
        # context['image'] = "Not Found"      
    
    # print(context)
        
    return JsonResponse(context, safe=False)



def updateStaff(request):

    if request.method != 'POST':
        messages.error(request, "Access Denied")
        return redirect(reverse('staffDirectory'))
        
    try:
        staff_id = request.POST.get('id')
        #print(staff_id)
        staff = Staff.objects.filter(id=staff_id).last()
        #print(staff.description)
        form = StaffForm(request.POST or None, instance=staff)
        if form.is_valid():
            form.save()
            
            messages.success(request, "Staff Updated Successfully")
        else:
            messages.error(request, form.errors)
    except:
        messages.error(request, "Access To This Resource Denied")
    
    
    return redirect(reverse('staffDirectory'))




def deleteStaff(request):
    if request.method != "POST":
        messages.error(request, "Access Denied")

    staff = Staff.objects.filter(id=request.POST.get('id')).last()
    # print(staff)
    if staff:
        try:   
            staff.delete()
            messages.success(request, "Staff deleted successfully")
        except:
            messages.error(request, "Sorry, cannot delete this staff")
    else:
        messages.error(request, "Staff does not exist")
    
    return redirect(reverse("staffDirectory"))





def viewStaffById(request):
    id = request.GET.get('id')
    # print(id)
    staff = Staff.objects.filter(id=id).last()
    # print(staff)
    context = {}
    if staff:
        context['name'] = staff.name
        # context['last_name'] = staff.last_name
        context['role'] = staff.role.id
        context['department'] = staff.department.id
        context['designation'] = staff.designation.id
        context['basic_salary'] = staff.basic_salary
        context['phone'] = staff.contact
        context['staff_id'] = staff.staff_id
        context['gender'] = staff.gender
        context['dob'] = staff.dob
        context['email'] = staff.email
        context['contact'] = staff.contact
        context['bank_account_number'] = staff.bank_account_number
        # context['account_name'] = staff.account_name
        context['bank_name'] = staff.bank_name
        context['bank_branch'] = staff.bank_branch
        context['address'] = staff.address
        context['id'] = staff.id
    else:
        return JsonResponse({'Not Found': 'Not Found'}, status=404)
    
    # print(context)
    return JsonResponse(context, safe=False, status=200)







def updateDepartment(request):

    if request.method != 'POST':
        messages.error(request, "Access Denied")
        return redirect(reverse('staffDirectory'))
        
    try:
        department_id = request.POST.get('id')
        #print(department_id)
        department = Department.objects.filter(id=department_id).last()
        #print(department.description)
        form = DepartmentForm(request.POST or None, instance=department)
        if form.is_valid():
            form.save()
            
            messages.success(request, "Department Updated Successfully")
        else:
            messages.error(request, "Department Not Updated")
    except:
        messages.error(request, "Access To This Resource Denied")
    
    
    return redirect(reverse('department'))






def deleteDepartment(request):
    if request.method != "POST":
        messages.error(request, "Access Denied")

    department = Department.objects.filter(id=request.POST.get('id')).last()
    # print(department)
    if department:
        try:   
            department.delete()
            messages.success(request, "Department deleted successfully")
        except:
            messages.error(request, "Sorry, cannot delete this department")
    else:
        messages.error(request, "Department does not exist")
    
    return redirect(reverse("department"))





def viewDepartmentById(request):
    id = request.GET.get('id')
    # print(id)
    department = Department.objects.filter(id=id).last()
    # print(staff)
    context = {}
    if department:
        context['department'] = department.department
        context['id'] = department.id
    else:
        return JsonResponse({'Not Found': 'Not Found'}, status=404)
    
    # print(context)
    return JsonResponse(context, safe=False, status=200)






def updateDesignation(request):

    if request.method != 'POST':
        messages.error(request, "Access Denied")
        return redirect(reverse('staffDirectory'))
        
    try:
        designation_id = request.POST.get('id')
        #print(Designation_id)
        designation = Designation.objects.filter(id=designation_id).last()
        #print(Designation.description)
        form = DesignationForm(request.POST or None, instance=designation)
        if form.is_valid():
            form.save()
            
            messages.success(request, "Designation Updated Successfully")
        else:
            messages.error(request, "Designation Not Updated")
    except:
        messages.error(request, "Access To This Resource Denied")
    
    
    return redirect(reverse('designation'))






def deleteDesignation(request):
    if request.method != "POST":
        messages.error(request, "Access Denied")

    designation = Designation.objects.filter(id=request.POST.get('id')).last()
    # print(designation)
    if designation:
        try:   
            designation.delete()
            messages.success(request, "Designation deleted successfully")
        except:
            messages.error(request, "Sorry, cannot delete this designation")
    else:
        messages.error(request, "Designation does not exist")
    
    return redirect(reverse("designation"))





def viewDesignationtById(request):
    id = request.GET.get('id')
    # print(id)
    designation = Designation.objects.filter(id=id).last()
    # print(staff)
    context = {}
    if designation:
        context['designation'] = designation.designation
        context['id'] = designation.id
    else:
        return JsonResponse({'Not Found': 'Not Found'}, status=404)
    
    # print(context)
    return JsonResponse(context, safe=False, status=200)



def printPayroll(request):
    
    YEAR_CHOICES = [ r for r in range(datetime.date.today().year-1, datetime.date.today().year+1)]
    
    context = { 'YEAR_CHOICES':YEAR_CHOICES}
    
    
    return render(request, 'printPayroll.html', context)



def printPayrollData(request):
     
    data = json.loads(request.body)

    month = data['month']
    year = data['year']
    
    # staff = Staff.objects.filter(role=role)
    # print(month, year)
    payroll = Payroll.objects.filter(month=month, year=year)
    # print(payroll)
    print_payslip = []
    
    for pay in payroll:
        # print(pay.staff.bank_account_number)
        payslip = [pay.staff.staff_id, pay.staff.name, pay.staff.bank_account_number, pay.staff.bank_name, pay.staff.bank_branch, pay.net_salary]
        print_payslip.append(payslip)
    # print(print_payslip)
    return JsonResponse(list(print_payslip), safe=False)




def printPayslip(request):
     
    data = json.loads(request.body)

    month = data['month']
    year = data['year']
    
    # print(month, year)
    payroll = Payroll.objects.filter(month=month, year=year)
    # print(payroll)
    print_payslip = []
    
    for pay in payroll:
        # print(pay.staff.bank_account_number)
        payslip = [pay.staff.staff_id, pay.staff.name, pay.staff.bank_account_number, pay.staff.bank_name, pay.staff.bank_branch, pay.net_salary]
        print_payslip.append(payslip)
    print('Im here')
    

    return JsonResponse(list(print_payslip), safe=False)
    
    




def individualPayroll(request):
    
    YEAR_CHOICES = [ r for r in range(datetime.date.today().year-1, datetime.date.today().year+1)]
    staff = Staff.objects.all()
    
    
    context = { 'YEAR_CHOICES':YEAR_CHOICES, 'staff':staff}
    
    return render(request, 'individualPayroll.html', context)





def individualPayrollData(request):
     
    data = json.loads(request.body)

    month = data['month']
    year = data['year']
    staff = data['staff']
    
    payroll = Payroll.objects.filter(month=month, year=year, staff=staff).first()
    
    context = {}
    context['name'] = payroll.staff.name
    context['staff_id'] = payroll.staff.staff_id
    context['role'] = payroll.staff.role.role
    context['department'] = payroll.staff.department.department
    context['basic_salary'] = payroll.basic_salary
    context['bonus'] = decimal.Decimal(payroll.bonus) + decimal.Decimal(payroll.allowance)
    context['deduction'] = payroll.deduction
    context['total_deductions'] = payroll.total_deductions
    context['net_salary'] = payroll.net_salary
    context['month'] = payroll.month
    context['year'] = payroll.year
    context['account_number'] = payroll.staff.bank_account_number
    context['bank_name'] = payroll.staff.bank_name
    
    # print(context)
    return JsonResponse(context, status=200)